# -*- coding: utf8 -*-
#!/usr/bin/env  python
# ===================================
# by Ph. MARON, 12th April 2012
# =================================== 
import h5py
from numpy import array, zeros, ones, ndarray
import datetime
from datetime import date, time
from matplotlib.dates import date2num
import time

def str2(i):
    if i < 10:
        str2='0'+str(int(i))
    else:
        str2=str(int(i))
    return str2

def str5(i):
    if i<10:
        str5='0000'+str(int(i))
    elif i<100:
        str5='000'+str(int(i))
    elif i<1000:
        str5='00'+str(int(i))    
    elif i<10000:
        str5='0'+str(int(i))    
    elif i<100000:
        str5=str(int(i))    
    return str5

class Hdf5(object):
    """ 
        Define a class of object associated with .hdf5 file 
        it works only for file which datasets are on the firs, second or third 
        level of the structure such as following :
        file ___ dataset 0
               !
               !_ group 1 __ dataset 1.1  
               !               !_ dataset 1.2
               !               :
               !               !_ dataset 1.n
               !               
               !_ group 2 ___ group 2.1 ___ dataset 2.1.1  
               !                 !                     !_ dataset 2.1.2
               !                 !                     :
               !                 !                      !_ dataset 2.1.n
               !                 !               
               !                 !__ group 2.2 ___ dataset 2.2.1  
               !                 !                      !_ dataset 2.2.2
               !                 !                      :
               !                 !                      !_ dataset 2.2.n  
               :                 :                      
               :                 !__ group 2.k ___ dataset 2.k.1  
               :                 :                      !_ dataset 2.k.2
               :                 :                      :
               :                 :                      !_ dataset 2.k.n 
   
        you can obtained the data include in each dataset as array by writing
        'file.keyword' in wich keyword correspond exactly to the name of dataset 0,
        group 1, group 2.1, groupe 2.2, groupe 2.k written with lower case letters 
        and underscor '_' instead of space. for example : 'Velocity U' give 'velocity_u'
        
        'file.keyword' gives 2d array (x, y ), 3D array (t,x,y) or 4D array ( t, z, x, y)
        _____________________
        
        .name returns the name of the file
        
        .nbtime returns the number of time values recorded in the file 
            ex : 6 

        .treediagram returns a list containing the structure of the file
        
        .nbfield returns the number of fields data contained in the file
        
        .time gives an array of the data of time recorded in the file
            ex : [[ 2010.     4.     7.    12.     0.     0.]  [ 2010.     4.     8.    12.     0.     0.]
                    ... [ 2010.     4.    13.    12.     0.     0.]]
            
        .year returns an array of the year values in format 'YYYY' 
            ex : [2010  2010   ...  2010]           
            
        .month returns an array of the month values in format 'MM' 
            ex : [4  4  ...  4]             
           
        .day returns an array of the date values in format 'DD' 
            ex : [7  8  ... 13] 
 
        .hour returns an array of the hour values in format 'HH' 
            ex : [12  12  ...  12] 
            
        .minute returns an array of the minute values in format 'MM' 
            ex : [0  0  ...  00] 
            
        .second returns an array of the second values in format 'SS' 
            ex : [0  0  ...  00] 
            
        .date gives a list of the data of dates recorded in the file as 
            ex : [ '2010-4-7', '2010-4-8', ..., '2010-4-13']

        .timeh gives a list of the data of time recorded in the file as 
            ex : [[ '12:00:00', 12:00:00', ..., 12:00:00']

        .datetime returns a list of the time values in format 'YYYY-MM-DD hh:mm:ss' 
            ex : ['2004-01-07 12:00:00',''2004-01-08 12:00:00'', ..., '2004-01-13 12:00:00'] 

        .filetime returns a list of the time values in format 'YYYYMMDDhhmmss' 
            ex : ['20040107120000',''20040108120000'', ..., '20040113120000'] 
            
        .date2num returns a list of the date values in date2num format 
            ex : [732092.3333, 732092.3403, ....]            
        _____________________
        for the field of data like 'latitude', 'longitude', 'connectionX', 'connectionY', 'verticalZ'
        giving the location of the grid caracteristic points, it is possible to obtain directly
        the location of the grid center points by keyword .coordlat, .coordlon,.coordx, .coordy 
        coordz

        """
 
    def __init__(self,name):
        self.name=name
        self.read_nbtime()
        self.read_time()
        self.read_treediagram()   
        self.close()

    def close(self):
        h5file = h5py.File(self.name,'r')
        h5file.close()
  
    def read_nbtime(self):
        h5file = h5py.File(self.name,'r')
        if 'Time' in h5file.keys():
            time=h5file['/Time']
            nbtime=len(time)
        else :
            print ' ERROR : no "Time" fields in this file '
        h5file.close()
        self.nbtime=nbtime

    def read_time(self):
        h5file = h5py.File(self.name,'r')
        arraytime=self.get_group('Time')
        h5file.close()
        self.year=arraytime[:,0]
        self.month=arraytime[:,1]
        self.day=arraytime[:,2]
        self.hour=arraytime[:,3]
        self.minute=arraytime[:,4]
        self.second=arraytime[:,5]
        

    def read_treediagram(self):
        h5file = h5py.File(self.name,'r')
        namelist=[]
        for i in range(len(h5file.keys())):
            level1name='/'+h5file.keys()[i]
            level1=h5file[level1name]
            type=str(h5file.items()[i][1]).split()[1]       
            namelist.append([level1.name,type])
            if type=='group':
                for j in range(len(level1.keys())):
                    level2name='/'+level1.keys()[j]
                    level2=h5file[level1.name+'/'+level2name]
                    type=str(level1.items()[j][1]).split()[1]       
                    namelist.append([level2.name,type])
                    if type=='group':
                        for k in range(len(level2.keys())):
                            level3name='/'+level2.keys()[k]
                            level3=h5file[level2.name+'/'+level3name]
                            type=str(level2.items()[k][1]).split()[1]       
                            namelist.append([level3.name,type])
                            if type=='group':
                                for l in range(len(level3.keys())):
                                    level4name='/'+level3.keys()[l]
                                    level4=h5file[level3.name+'/'+level4name]
                                    type=str(level3.items()[l][1]).split()[1]       
                                    namelist.append([level4.name,type])
        fields=[]
        for lig in namelist:
            fieldnames=lig[0].split('/')
            if '_' not in fieldnames[-1]:
                # if fieldnames[-1] not in h5file.keys():
                    fields.append([lig[0], fieldnames[-1].lower().replace(' ','_'), lig[1]])
        h5file.close()
        self.treediagram=array(fields)
        self.nbfields=len(fields)
        
    def __getattr__(self, name):
        if name in self.__dict__:
            return self.__dict__[name]
        else:
            h5file = h5py.File(self.name,'r')
            #read field which names are in the structure of the file 
            for field in self.treediagram:
                if name==field[1] and field[2]=='dataset':
                    value=self.get_dataset(field[0])
                    setattr(self, name, value)
                    return value
                elif name==field[1] and field[2]=='group':
                    value=self.get_group(field[0])
                    setattr(self, name, value)
                    return value
            # read and modify date format
            if name in ['date', 'timeh','datetime','date2num','filetime']:
                date_=[]
                for i in range(self.nbtime):
                    if name=='date' :
                        date_.append(str(datetime.date(int(self.year[i]),int(self.month[i]),int(self.day[i]))))
                    elif name=='timeh':
                        date_.append(str(datetime.time(int(self.hour[i]),int(self.minute[i]),int(self.second[i]))))
                    elif name=='datetime':
                        date_.append(str(datetime.datetime(int(self.year[i]),int(self.month[i]),int(self.day[i]),int(self.hour[i]),int(self.minute[i]),int(self.second[i]))))              
                    elif name=='date2num':
                        date_.append(date2num(datetime.datetime(int(self.year[i]),int(self.month[i]),int(self.day[i]),int(self.hour[i]),int(self.minute[i]),int(self.second[i]))))              
                    elif name=='filetime':
                        date_.append(str(int(self.year[i]))+str2(self.month[i])+str2(self.day[i])+str2(self.hour[i])+str2(self.minute[i])+str2(self.second[i]))
                value = date_
                h5file.close()
                return value
            #calculate grid center points location
            if name in ['coordlat', 'coordlon', 'coordx', 'coordy', 'coordz']:
                if name =='coordlat':
                    value = self.get_coord(self.latitude)
                if name =='coordlon':
                    value = self.get_coord(self.longitude)
                if name =='coordx':
                    value = self.get_coord(self.connectionx)
                if name =='coordy':
                    value = self.get_coord(self.connectiony)
                if name =='coordz':
                    value = self.get_coordz(self.verticalz)
                setattr(self, name, value) 
                return value

    def get_coordz(self,name):
        data=name.copy()
        coordata=ndarray((data.shape[0],data.shape[1]-1,data.shape[2],data.shape[3]))
        for t in range(data.shape[0]):
            for i in range(data.shape[2]):
                for j in range(data.shape[3]):
                    for k in range(data.shape[1]-1):
                        coordata[t,k,i,j]=(data[t,k,i,j]+data[t,k+1,i, j])/2.
        return coordata
    
    def get_coord(self,name):
        data=name.copy()
        coordata=ndarray((data.shape[0]-1,data.shape[1]-1))
        for i in range(data.shape[0]-1):
            for j in range(data.shape[1]-1):
                coordata[i,j]=(data[i,j]+data[i+1, j+1])/2.
        return coordata
                    
    def get_dataset(self,id):
        h5file = h5py.File(self.name,'r')
        value=h5file[id]
        return array(value)
        
    def get_group(self,id):
        h5file = h5py.File(self.name,'r')
        valueg=[]
        for t in range(self.nbtime):
            # incoherence de notation sur VerticalZ 
            if id.split('/')[-1]=='VerticalZ':
                idt = id+'/Vertical_'+str5(t+1)
            else:    
                idt = id+'/'+id.split('/')[-1]+'_'+str5(t+1)
            value=h5file[idt]
            valueg.append(value)
        return array(valueg) 

# n=raw_input('Type to follow !')
    
        
if __name__=='__main__' :
    # filename='MERCATOR.hdf5'
    filename='hdf5files.Hdf5_Mercator_3D.hdf5'
    # filename='Fileshdf5.Mercator_2D.hdf5'

    print '======================================================================'
    print ' Test on '+filename
    print '======================================================================'
    file = Hdf5(filename)
    print 'file.name :', file.name
    print 'file.nbtime : ', file.nbtime
    print 'file.treediagram : ', file.treediagram, file.treediagram.shape
    print 'file.nbfields : ', file.nbfields
    n=raw_input('Type to follow with '+filename+' !')
    print 'file.bathymetry : ',file.bathymetry,file.bathymetry.shape
    print 'file.connectionx : ', file.connectionx, file.connectionx.shape
    print 'file.coordx : ',file.coordx, file.coordx.shape
    print 'file.connectiony : ', file.connectiony, file.connectiony.shape
    print 'file.coordy : ',file.coordy, file.coordy.shape
    print 'file.latitude : ', file.latitude[0,:], file.latitude.shape
    print 'file.coordlat : ', file.coordlat[0,:], file.coordlat.shape
    print 'file.longitude : ',file.longitude, file.longitude.shape
    print 'file.coordlon : ', file.coordlon,file.coordlon.shape
    print 'file.verticalz : ', file.verticalz[1,:,10,30],file.verticalz.shape
    print 'file.coordz : ', file.coordz[1,:,10,30],file.coordz.shape
    print 'file.verticalz : ',file.verticalz.shape
    print 'file.coordz : ', file.coordz.shape
    print 'file.waterpoints3d : ', file.waterpoints3d, file.waterpoints3d.shape
    print 'file.barotropic_velocity_u : ',file.barotropic_velocity_u, file.barotropic_velocity_u.shape
    print 'file.barotropic_velocity_v : ',file.barotropic_velocity_v, file.barotropic_velocity_v.shape
    print 'file.salinity : ',file.salinity, file.salinity.shape
    print 'file.temperature : ',file.temperature, file.temperature.shape
    print 'file.velocity_u : ',file.velocity_u, file.velocity_u.shape
    print 'file.velocity_v : ',file.velocity_v, file.velocity_v.shape
    print 'file.velocity_modulus : ',file.velocity_modulus, file.velocity_modulus.shape
    print 'file.water_level : ',file.water_level, file.water_level.shape
    print 'file.time : ', file.time, file.time.shape
    print 'file.year : ', file.year
    print 'file.month : ',file.month
    print 'file.day : ',file.day
    print 'file.hour : ',file.hour
    print 'file.minute : ',file.minute
    print 'file.second : ',file.second
    print'file.date : ', file.date    
    print'file.timeh : ', file.timeh 
    print'file.datetime : ', file.datetime   
    print 'file.filetime : ', file.filetime
    print'file.date2num : ', file.date2num
        
    print '======================================================================'
    print ' End of test on '+filename   
    print '======================================================================'
    
 
    filename='hdf5files.Hdf5_MeteoGalicia.hdf5'
    # print time.strftime('%d/%m/%y %H:%M:%S',time.localtime())  
    n=raw_input('Type to follow with '+filename+' !')



    print '======================================================================'
    print ' Test on '+filename
    print '======================================================================'
    file = Hdf5(filename)
    print 'file.name :', file.name
    print 'file.nbtime : ', file.nbtime
    print 'file.treediagram : ', file.treediagram, file.treediagram.shape
    print 'file.nbfields : ', file.nbfields
    print 'file.bathymetry : ',file.bathymetry,file.bathymetry.shape
    # print 'file.connectionx : ', file.connectionx, file.connectionx.shape
    # print 'file.coordx : ',file.coordx, file.coordx.shape
    # print 'file.connectiony : ', file.connectiony, file.connectiony.shape
    # print 'file.coordy : ',file.coordy, file.coordy.shape
    # print 'file.latitude : ', file.latitude[0,:], file.latitude.shape
    # print 'file.coordlat : ', file.coordlat[0,:], file.coordlat.shape
    # print 'file.longitude : ',file.longitude, file.longitude.shape
    # print 'file.coordlon : ', file.coordlon,file.coordlon.shape
    print 'file.air_temperature', file.air_temperature
    print 'file.air_temperature', file.air_temperature.shape
    # print file.downward_long_wave_radiation
    # print file.latent_heat
    # print file.mean_sea_level_pressure
    # print file.precipitation
    # print file.relative_humidity
    # print file.sea_water_temperature
    # print file.sensible_heat
    # print file.solar_radiation
    # print file.wind_direction
    # print file.wind_modulus
    # print file.wind_velocity_x
    # print file.wind_velocity_y
    # print 'file.time : ', file.time, file.time.shape
    # print 'file.year : ', file.year
    # print 'file.month : ',file.month
    # print 'file.day : ',file.day
    # print 'file.hour : ',file.hour
    # print 'file.minute : ',file.minute
    # print 'file.second : ',file.second
    # print'file.date : ', file.date    
    # print'file.timeh : ', file.timeh 
    # print'file.datetime : ', file.datetime   
    # print'file.date2num : ', file.date2num
        
    print '======================================================================'
    print ' End of test on '+filename
    print '======================================================================'
  
#n=raw_input('Type to close !')



